<?php
namespace Migration\Step\AheadworksRma;

use Migration\App\Step\StageInterface;
use Migration\Reader\MapInterface;
use Migration\Reader\GroupsFactory;
use Migration\Reader\Map;
use Migration\Reader\MapFactory;
use Migration\ResourceModel;
use Migration\ResourceModel\Record;
use Migration\App\ProgressBar;
use Migration\Logger\Manager as LogManager;
use Migration\Logger\Logger;
use Migration\Config;
use Migration\ResourceModel\Document;
use Migration\Step\AheadworksRma\Helper\CustomField;
use Migration\Step\AheadworksRma\Helper\Request as RequestHelper;
use Migration\Step\AheadworksRma\Helper\Thread as ThreadHelper;

/**
 * Class Data
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class Data extends \Migration\Step\DatabaseStage implements StageInterface
{
    /**
     * @var ResourceModel\Source
     */
    private $source;

    /**
     * @var ResourceModel\Destination
     */
    private $destination;

    /**
     * @var ResourceModel\RecordFactory
     */
    private $recordFactory;

    /**
     * @var Map
     */
    private $map;

    /**
     * @var \Migration\RecordTransformerFactory
     */
    private $recordTransformerFactory;

    /**
     * @var ProgressBar\LogLevelProcessor
     */
    private $progress;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var Helper
     */
    private $helper;

    /**
     * @var \Migration\Reader\Groups
     */
    private $readerGroups;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var bool
     */
    private $removeRequests;

    /**
     * @var CustomField
     */
    private $customFieldHelper;

    /**
     * @var RequestHelper
     */
    private $requestHelper;

    /**
     * @var ThreadHelper
     */
    private $threadHelper;

    /**
     * Data constructor.
     * @param ProgressBar\LogLevelProcessor $progress
     * @param ResourceModel\Source $source
     * @param ResourceModel\Destination $destination
     * @param ResourceModel\RecordFactory $recordFactory
     * @param \Migration\RecordTransformerFactory $recordTransformerFactory
     * @param MapFactory $mapFactory
     * @param GroupsFactory $groupsFactory
     * @param Logger $logger
     * @param Helper $helper
     * @param CustomField $customFieldHelper
     * @param RequestHelper $requestHelper
     * @param ThreadHelper $threadHelper
     * @param Config $config
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        ProgressBar\LogLevelProcessor $progress,
        ResourceModel\Source $source,
        ResourceModel\Destination $destination,
        ResourceModel\RecordFactory $recordFactory,
        \Migration\RecordTransformerFactory $recordTransformerFactory,
        MapFactory $mapFactory,
        GroupsFactory $groupsFactory,
        Logger $logger,
        Helper $helper,
        CustomField $customFieldHelper,
        RequestHelper $requestHelper,
        ThreadHelper $threadHelper,
        Config $config
    ) {
        $this->source = $source;
        $this->destination = $destination;
        $this->recordFactory = $recordFactory;
        $this->recordTransformerFactory = $recordTransformerFactory;
        $this->map = $mapFactory->create('aw_rma_map_file');
        $this->progress = $progress;
        $this->readerGroups = $groupsFactory->create('aw_rma_groups_file');
        $this->logger = $logger;
        $this->helper = $helper;
        $this->config = $config;
        $this->customFieldHelper = $customFieldHelper;
        $this->requestHelper = $requestHelper;
        $this->threadHelper = $threadHelper;
        $this->removeRequests = (bool) $this->config->getOption('aw_rma_remove_requests_before_migration');
        parent::__construct($config);
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function perform()
    {
        $destinationAdapter = $this->destination->getAdapter()->getSelect()->getAdapter();
        $sourceDocuments = array_keys($this->readerGroups->getGroup('source_documents'));

        //Clear all data related to requests if needed
        if ($this->removeRequests) {
            $this->requestHelper->clearRequestData();
            $this->threadHelper->clearThreadData();
        }

        // Migrate RMA package condition options
        $this->customFieldHelper->preparePackageConditionCustomField();

        $this->progress->start(count($sourceDocuments), LogManager::LOG_LEVEL_INFO);
        foreach ($sourceDocuments as $sourceDocName) {
            $sourceDocument = $this->source->getDocument($sourceDocName);
            $destinationName = $this->map->getDocumentMap($sourceDocName, MapInterface::TYPE_SOURCE);
            if (!$destinationName) {
                continue;
            }
            $destDocument = $this->destination->getDocument($destinationName);

            $this->helper->setDestinationCount($destinationName);
            $this->logger->debug('migrating', ['table' => $sourceDocName]);
            $recordTransformer = $this->getRecordTransformer($sourceDocument, $destDocument);
            $pageNumber = 0;
            $this->progress->start(
                ceil($this->source->getRecordsCount($sourceDocName) / $this->source->getPageSize($sourceDocName)),
                LogManager::LOG_LEVEL_DEBUG
            );
            while (!empty($items = $this->source->getRecords($sourceDocName, $pageNumber))) {
                $pageNumber++;
                foreach ($items as $recordData) {
                    $this->source->setLastLoadedRecord($sourceDocName, $recordData);
                    /** @var Record $record */
                    $record = $this->recordFactory->create(['document' => $sourceDocument, 'data' => $recordData]);
                    /** @var Record $destRecord */
                    $destRecord = $this->recordFactory->create(['document' => $destDocument]);

                    $this->progress->advance(LogManager::LOG_LEVEL_DEBUG);

                    // Migrate RMA entity type options
                    if ($sourceDocName == 'aw_rma_entity_types') {
                        $this->customFieldHelper->saveCustomField($record, CustomField::RESOLUTION_ID);
                        continue;
                    }
                    // Migrate RMA entity reason options
                    if ($sourceDocName == 'aw_rma_entity_reason') {
                        $this->customFieldHelper->saveCustomField($record, CustomField::REASON_ID);
                        continue;
                    }

                    $recordTransformer->transform($record, $destRecord);
                    if ($destRecord->getValue('id')) {
                        $destRecord->setValue('id', null);
                    }

                    $destinationAdapter->insertMultiple(
                        $this->destination->addDocumentPrefix($destinationName),
                        $destRecord->getData()
                    );

                    $insertId = $destinationAdapter->lastInsertId();

                    // Migrate additional data of RMA requests
                    $customFieldsOldNewData = $this->customFieldHelper->getCustomFieldOldNewData();
                    if ($sourceDocName == 'aw_rma_entity') {
                         $this->requestHelper->saveRequest($record, $insertId, $customFieldsOldNewData);
                    }

                    // Migrate RMA thread attachments
                    if ($sourceDocName == 'aw_rma_entity_comments') {
                        $this->threadHelper->saveAttachments($record, $insertId);
                    }
                }
            }
            $this->source->setLastLoadedRecord($sourceDocName, []);
            $this->progress->advance(LogManager::LOG_LEVEL_INFO);
            $this->progress->finish(LogManager::LOG_LEVEL_DEBUG);
        }
        $this->progress->finish(LogManager::LOG_LEVEL_INFO);
        return true;
    }

    /**
     * Get record transformer
     *
     * @param Document $sourceDocument
     * @param Document $destDocument
     * @return \Migration\RecordTransformer
     */
    public function getRecordTransformer(Document $sourceDocument, Document $destDocument)
    {
        /** @var \Migration\RecordTransformer $recordTransformer */
        $recordTransformer = $this->recordTransformerFactory->create(
            [
                'sourceDocument' => $sourceDocument,
                'destDocument' => $destDocument,
                'mapReader' => $this->map
            ]
        );
        $recordTransformer->init();
        return $recordTransformer;
    }
}
