<?php
namespace Migration\Step\AheadworksRma\Helper;

use Migration\ResourceModel\Destination;
use Migration\Reader\GroupsFactory;
use Migration\Config;
use Magento\Store\Model\Store;
use Migration\Step\AheadworksRma\Helper;
use Migration\ResourceModel\Record;

/**
 * Class CustomField
 * @package Migration\Step\AheadworksRma\Helper
 */
class CustomField extends Helper
{
    const RESOLUTION_ID = 1;
    const PACKAGE_CONDITION_ID = 2;
    const REASON_ID = 3;

    /**
     * @var string
     */
    private $customFieldTable = 'aw_rma_custom_field';

    /**
     * @var string
     */
    private $customFieldOptionTable = 'aw_rma_custom_field_option';

    /**
     * @var string
     */
    private $customFieldOptionValueTable = 'aw_rma_custom_field_option_value';

    /**
     * @var []
     */
    private $collectCustomFieldOldNewData = [];

    /**
     * @var []
     */
    private $tableToCollectCustomFieldOldNewData = [];

    /**
     * @param GroupsFactory $groupsFactory
     * @param Destination $destination
     * @param Config $config
     */
    public function __construct(
        GroupsFactory $groupsFactory,
        Destination $destination,
        Config $config
    ) {
        parent::__construct($groupsFactory, $destination, $config);
        $this->tableToCollectCustomFieldOldNewData = $this->readerGroups->getGroup('collect_custom_field_old_new_data');
    }

    /**
     * Get field from M1 RMA by custom field ID in M2
     *
     * @param $customFieldId
     * @return string
     */
    public function getMappedFieldNameByCustomFieldId($customFieldId)
    {
        $m1m2FieldMapping = [
            self::RESOLUTION_ID => 'request_type',
            self::PACKAGE_CONDITION_ID => 'package_opened',
            self::REASON_ID => 'reason_id'
        ];

        return $m1m2FieldMapping[$customFieldId];
    }

    /**
     * Check for existence package conditions in M2 custom field
     */
    public function preparePackageConditionCustomField()
    {
        $yesConditionOption = [
            'id' => '1',
            'name' => 'Opened',
            'store' => '0',
            'sort' => '1',
            'enabled' => '1'
        ];

        $this->saveCustomFieldOption($yesConditionOption, self::PACKAGE_CONDITION_ID);

        $noConditionOption = [
            'id' => '0',
            'name' => 'Not opened',
            'store' => '0',
            'sort' => '1',
            'enabled' => '1'
        ];

        $this->saveCustomFieldOption($noConditionOption, self::PACKAGE_CONDITION_ID);
    }

    /**
     * Save data of custom field
     *
     * @param Record $sourceRecord
     * @param int $customFieldId
     */
    public function saveCustomField($sourceRecord, $customFieldId)
    {
        $sourceData = $sourceRecord->getData();
        $this->saveCustomFieldOption($sourceData, $customFieldId);
    }

    /**
     * Save data of custom field option
     *
     * @param array $sourceData
     * @param int $customFieldId
     */
    public function saveCustomFieldOption($sourceData, $customFieldId)
    {
        if (!$this->checkIfOptionExists($sourceData, $customFieldId)) {
            $customFieldOptionData = [
                'field_id' => $customFieldId,
                'sort_order' => $sourceData['sort'],
                'is_default' => 0,
                'enabled' => $sourceData['enabled']
            ];
            $lastInsertId = $this->makeInsert($this->customFieldOptionTable, 'id', $customFieldOptionData);
            $this->saveCustomFieldOptionValue($lastInsertId, $sourceData, $customFieldId);

        }
    }

    /**
     * Save custom field option store value and default value
     *
     * @param int $optionId
     * @param array $sourceData
     * @param int $customFieldId
     */
    public function saveCustomFieldOptionValue($optionId, $sourceData, $customFieldId)
    {
        $customFieldOptionValueStoreData = [
            'option_id' => $optionId,
            'store_id' => $sourceData['store'],
            'value' => $sourceData['name']
        ];
        $this->makeInsert($this->customFieldOptionValueTable, 'option_id', $customFieldOptionValueStoreData);

        if ($sourceData['store'] != Store::DEFAULT_STORE_ID) {
            $customFieldOptionValueDefaultData = [
                'option_id' => $optionId,
                'store_id' => Store::DEFAULT_STORE_ID,
                'value' => $sourceData['name']
            ];
            $this->makeInsert($this->customFieldOptionValueTable, 'option_id', $customFieldOptionValueDefaultData);
        }

        $this->saveCustomFieldOldNewData($customFieldId, $sourceData['id'], $optionId);
    }

    /**
     * Check if custom field option already exists
     *
     * @param array $sourceData
     * @param int $customFieldId
     * @return bool
     */
    public function checkIfOptionExists($sourceData, $customFieldId)
    {
        $adapter = $this->destination->getAdapter();
        $query = $adapter->getSelect()
            ->from(
                ['custom_field' => $this->customFieldTable],
                []
            )->joinLeft(
                ['custom_field_option' => $this->customFieldOptionTable],
                'custom_field.id = custom_field_option.field_id',
                ['option_id' => 'custom_field_option.id']
            )->joinLeft(
                ['custom_field_option_value' => $this->customFieldOptionValueTable],
                'custom_field_option.id = custom_field_option_value.option_id',
                []
            )->where('custom_field.id = ?', $customFieldId)
            ->where('custom_field_option_value.store_id = ?', $sourceData['store'])
            ->where('custom_field_option_value.value = ?', $sourceData['name']);

        if ($optionId = $query->getAdapter()->fetchCol($query)) {
            $this->saveCustomFieldOldNewData($customFieldId, $sourceData['id'], $optionId[0]);
            return true;
        }
        return false;
    }

    /**
     * Save old new data about custom field options
     *
     * @param int $fieldId
     * @param int $oldId
     * @param int $optionId
     */
    public function saveCustomFieldOldNewData($fieldId, $oldId, $optionId)
    {
        if (isset($this->tableToCollectCustomFieldOldNewData[$this->getMappedFieldNameByCustomFieldId($fieldId)])) {
            $this->collectCustomFieldOldNewData[$this->getMappedFieldNameByCustomFieldId($fieldId)][$oldId] = $optionId;
        }
    }

    /**
     * Get old new data about custom field options
     *
     * @return array
     */
    public function getCustomFieldOldNewData()
    {
        return $this->collectCustomFieldOldNewData;
    }
}
