<?php
namespace Migration\Step\AheadworksRma\Helper;

use Migration\ResourceModel\Destination;
use Migration\Reader\GroupsFactory;
use Migration\ResourceModel\Record;
use Migration\Config;
use Migration\Step\AheadworksRma\Helper;
use Aheadworks\Rma\Model\Source\ThreadMessage\Owner;

/**
 * Class Request
 * @package Migration\Step\AheadworksRma\Helper
 */
class Request extends Helper
{
    /**
     * @var string
     */
    private $requestTable = 'aw_rma_request';

    /**
     * @var string
     */
    private $requestItemTable = 'aw_rma_request_item';

    /**
     * @var string
     */
    private $requestItemCustomFieldValueTable = 'aw_rma_request_item_custom_field_value';

    /**
     * @var string
     */
    private $requestCustomFieldValueTable = 'aw_rma_request_custom_field_value';

    /**
     * @var string
     */
    private $threadMessageTable = 'aw_rma_thread_message';

    /**
     * @var []
     */
    private $collectRequestOldNewData = [];

    /**
     * @var []
     */
    private $tableToCollectRequestOldNewData = [];

    /**
     * @var []
     */
    private $customFieldOldNewData = [];

    /**
     * @param GroupsFactory $groupsFactory
     * @param Destination $destination
     * @param Config $config
     */
    public function __construct(
        GroupsFactory $groupsFactory,
        Destination $destination,
        Config $config
    ) {
        parent::__construct($groupsFactory, $destination, $config);
        $this->tableToCollectRequestOldNewData = $this->readerGroups->getGroup('collect_request_old_new_data');
    }

    /**
     * Clear request table
     */
    public function clearRequestData()
    {
        $this->destination->clearDocument($this->requestTable);
        $this->destination->clearDocument($this->requestItemTable);
        $this->destination->clearDocument($this->requestItemCustomFieldValueTable);
        $this->destination->clearDocument($this->requestCustomFieldValueTable);
    }

    /**
     * Save old new data
     *
     * @param string $sourceDocName
     * @param Record $record
     * @param int $insertId
     * @return null
     */
    public function saveRequestOldNewData($sourceDocName, $record, $insertId)
    {
        if (isset($this->tableToCollectRequestOldNewData[$sourceDocName])) {
            $oldValue = $record->getValue($this->tableToCollectRequestOldNewData[$sourceDocName]);
            $this->collectRequestOldNewData[$sourceDocName][$oldValue] = $insertId;
        }
    }

    /**
     * Get old new data about requests
     * @return array
     */
    public function getRequestOldNewData()
    {
        return $this->collectRequestOldNewData;
    }

    /**
     * Save Request data
     *
     * @param Record $sourceRecord
     * @param int $requestId
     * @param array $customFieldOldNewData
     */
    public function saveRequest($sourceRecord, $requestId, $customFieldOldNewData)
    {
        $this->customFieldOldNewData = $customFieldOldNewData;
        $this->saveRequestOldNewData($this->requestTable, $sourceRecord, $requestId);
        $this->saveRequestItemData($sourceRecord, $requestId);
        $this->saveRequestCustomFieldData($sourceRecord, $requestId);
        $this->saveRequestInternalNote($sourceRecord, $requestId);
    }

    /**
     * Save request items
     *
     * @param Record $sourceRecord
     * @param int $requestId
     */
    public function saveRequestItemData($sourceRecord, $requestId)
    {
        $returnedItems = unserialize($sourceRecord->getValue('order_items'));
        foreach ($returnedItems as $itemId => $qty) {
            $requestItemData = [
                'item_id' => $itemId,
                'request_id' => $requestId,
                'qty' => $qty
            ];
            $lastInsertId = $this->makeInsert($this->requestItemTable, 'id', $requestItemData);

            if ($reasonId = $sourceRecord->getValue('reason_id')) {
                $value = $this->customFieldOldNewData['reason_id'][$reasonId];
                $customFieldId = CustomField::REASON_ID;
                $this->saveRequestItemCustomFieldValue($lastInsertId, $customFieldId, $value);
            }

        }
    }

    /**
     * Save request item custom field data
     *
     * @param int $entityId
     * @param int $customFieldId
     * @param int|string $value
     */
    public function saveRequestItemCustomFieldValue($entityId, $customFieldId, $value)
    {
        $requestItemCustomFieldValueData = [
            'entity_id' => $entityId,
            'field_id' => $customFieldId,
            'value' => $value
        ];
        $this->makeInsert($this->requestItemCustomFieldValueTable, 'entity_id', $requestItemCustomFieldValueData);
    }

    /**
     * Save request custom field data
     *
     * @param Record $sourceRecord
     * @param int $requestId
     */
    public function saveRequestCustomFieldData($sourceRecord, $requestId)
    {
        $requestType = $sourceRecord->getValue('request_type');
        $requestCustomFieldValueData = [
            'entity_id' => $requestId,
            'field_id' => CustomField::RESOLUTION_ID,
            'value' => $this->customFieldOldNewData['request_type'][$requestType]
        ];
        $this->makeInsert($this->requestCustomFieldValueTable, 'entity_id', $requestCustomFieldValueData);

        $packageOpened = $sourceRecord->getValue('package_opened');
        $requestCustomFieldValueData = [
            'entity_id' => $requestId,
            'field_id' => CustomField::PACKAGE_CONDITION_ID,
            'value' => $this->customFieldOldNewData['package_opened'][$packageOpened]
        ];
        $this->makeInsert($this->requestCustomFieldValueTable, 'entity_id', $requestCustomFieldValueData);
    }

    /**
     * Save internal note
     *
     * @param Record $sourceRecord
     * @param int $requestId
     */
    public function saveRequestInternalNote($sourceRecord, $requestId)
    {
        if ($internalNote = $sourceRecord->getValue('admin_notes')) {
            $internalNoteData = [
                'request_id' => $requestId,
                'created_at' => $sourceRecord->getValue('created_at'),
                'text' => $internalNote,
                'owner_type' => Owner::ADMIN,
                'owner_id' => 0,
                'is_auto' => 0,
                'is_internal' => 1
            ];
            $this->makeInsert($this->threadMessageTable, 'id', $internalNoteData);
        }
    }
}
